package com.jdbernard.nurseryschedule

import com.jdbernard.util.SmartConfig
import groovy.json.JsonBuilder
import groovy.json.JsonException
import groovy.json.JsonSlurper
import groovy.text.SimpleTemplateEngine
import java.util.Calendar
import java.util.regex.Matcher
import javax.servlet.ServletConfig
import javax.servlet.ServletException
import javax.servlet.http.HttpServlet
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse
import javax.servlet.http.HttpSession

import static java.util.Calendar.*
import static javax.servlet.http.HttpServletResponse.*

public class NurseryScheduleServlet extends HttpServlet {

    /// This service currently only supports one user.
    private String username
    private String password
    private String outputDirectory

    private static SimpleDateFormat dataDateFormat =
        new SimpleDateFormat("yyyy-MM-dd")
    private static SimpleDateFormat displayDateFormat =
        new SimpleDateFormat("MMM dd")

    void init(ServletConfig config) { 
        this.username = config.getInitParameter("username")
        this.password = config.getInitParameter("password")
        this.outputDirectory = config.getInitParameter("outputDirectory") }

    void doPost(HttpServletRequest request, HttpServletResponse response) {

        HttpSession session = request.getSession(true);

        /// If the user is posting to `/login` then let's try to authenticate
        /// them. We don't care about the state of the existing session.
        if (request.servletPath == '/login') {

            /// Parse the username/password from the request.
            def requestBody
            try { requestBody = new JsonSlurper().parse(request.reader) }
            catch (JsonException jsone) {
                response.status = SC_BAD_REQUEST
                return }

            /// If no username or password are configured we cannot
            /// authenticate the user.
            if (!password || !username) {
                response.status = SC_UNAUTHORIZED
                return }

            /// If the username or password do not match, deny the request.
            if (username != requestBody.username ||
                password != requestBody.password) {
                response.status = SC_UNAUTHORIZED
                return }

            response.status = SC_OK
            session.setAttribute('authenticated', 'true')
            session.setAttribute('username', requestBody.username)
            writeJSON([status: "ok"], response)
            return }

        /// The other endpoints require the user to be authenticated. 
        if (!((boolean)session.getAttribute('authenticated'))) {
            response.status = SC_UNAUTHORIZED
            return }

        switch (request.servletPath) {
            case ~ '/make-schedule':

                /// Parse the request body.
                def data
                try { data = new JsonSlurper().parse(request.reader) }
                catch (JsonException jsone) {
                    response.status = SC_BAD_REQUEST
                    return }

                /// Calculate dates and modify the schedule data to include
                /// them.
                def cal = Calendar.getInstance()
                cal.time = dataDateFormat.parse(data.date)
                int month = cal.get(MONTH)
                cal.set(HOUR_OF_DAY, 0)
                cal.set(MINUTE, 0)
                cal.set(SECOND, 0)
                cal.set(MILLISECOND, 0)

                ['first', 'second', 'third', 'fourth', 'fifth'].eachWithIndex
                { dayNum, idx ->
                    /// Set the sunday
                    cal.set(DAY_OF_WEEK, SUNDAY)
                    cal.set(DAY_OF_WEEK_IN_MONTH, idx + 1)
                    if (cal.get(MONTH) == month)
                        data.sundays[dayNum].date = displayDateFormat.format(cal.time)

                    /// Set the wednesday. Note that we have to reformat the
                    /// input data from {"first": ["PersonA", "PersonB"]} to
                    /// {"date": "Oct. 6", "volunteers": ["PersonA", PersonB"]}
                    def volunteers = data.wednesdays[dayNum]
                    data.wednesdays[dayNum] = [:]
                    data.wednesdays[dayNum].volunteers = volunteers

                    if (cal.get(MONTH) == month)
                        data.wednesdays[dayNum].date =
                            displayDateFormat.format(cal.time) }

                cal.set(DAY_OF_WEEK_IN_MONTH, 2)
                data.sundays['second'].date = displayDateFormat.format(cal.time)

                /// Build the resulting page.
                def engine = new SimpleTemplateEngine()
                def scheduleTemplate = engine.createTemplate(
                    this.getClass().getResource('schedule.template'))
                def output = scheduleTemplate.make(data)

                /// Write the results to file.
                new File(outputDirectory, data.date + ".html").text = output
                response.status = SC_OK

            default:
                response.status = SC_NOT_FOUND
                return
        }
    }
}
